#include "atnd.h"

extern char *queue[QUEUE_ARRAY_SIZE];
extern int queue_begin, queue_end;
extern int ring_count;

static char BELL_LF[3] = {0x07, 0x0A, 0};
static const int BELL_LF_SIZE = 2;

BOOL setting(RING_BUFFER *r, char *f, int a, int before, int p, int b, BOOL n) {
    BOOL allocated = TRUE;

    r->after = a;
    strcpy(r->file_name, f);
    r->number = n;
    r->point = p;
    r->ring_count = before + 1 + r->after;
    r->buffer_size = b;
    r->buffer = malloc(r->buffer_size);
    if(r->buffer == NULL) {
        fprintf(stderr, "malloc() error-3: %d bytes\n", r->buffer_size);
        allocated = FALSE;
    }

    r->combo = malloc(r->buffer_size);
    if(r->combo == NULL) {
        fprintf(stderr, "r->combo == NULL error: %d bytes\n",  r->buffer_size);
        allocated = FALSE;
    }

    return allocated;
}

BOOL reading(RING_BUFFER *r) {
    BOOL neighborhood = FALSE;
    BOOL written = FALSE;
    char *p;
    int count = 0;
    int line_number = 0;
    int sum = 0;
    LINE_NUMBER_FUNCTION line_begins = nonnumber;
    FILE *file= fopen(r->file_name, "rt");
    if(file == NULL) {
        error_handler("fopen() error: %s\n", r->file_name);
        return neighborhood;
    }

    ring_count = r->ring_count;
    prepare_ring();

    if(r->number) {
        line_begins = line_begins_with_number;
    }

    if(r->after > 0) {
        for(;;) {
            p = fgets(r->buffer, r->buffer_size, file);
            if(p == NULL) {
                break;
            }

            line_begins(r->buffer, r->combo);
            pop_front();
            push_back(r->buffer, r->buffer_size);

            if(neighborhood) {
                count += 1;
                if(count >= r->after) {
                    show_ring();
                    prepare_ring();
                    written = TRUE;
                    count = 0;
                    neighborhood = FALSE;
                }

            }

            line_number += 1;
            if(line_number == r->point) {
                neighborhood = TRUE;
                sum += 1;
            }

        }

    } else {
        for(;;) {
            p = fgets(r->buffer, r->buffer_size, file);
            if(p == NULL) {
                break;
            }

            line_begins(r->buffer, r->combo);
            pop_front();
            push_back(r->buffer, r->buffer_size);

            line_number += 1;
            if(line_number == r->point) {
                neighborhood = TRUE;
                sum += 1;
                show_ring();
                prepare_ring();
                written = TRUE;
            }

        }

    }

    if(neighborhood) {
        if(written) {
            ;
        } else {
            show_ring();
            written = TRUE;
        }

    } else {
        neighborhood = (sum > 0) ? TRUE : FALSE;
    }

    fclose(file);
    return neighborhood;
}

void error_handler(const char *p, char *q) {
    FILE *file = fopen(ERROR_TEXT, "wt");
    fprintf(file, "%s %s", p, q);;
    fclose(file);
}

void line_begins_with_number(char *buffer, char *combo) {
    static int line_number;

    line_number += 1;
    sprintf(combo, "%10d|%s", line_number, buffer);
    strcpy(buffer, combo);
}

void nonnumber(char *buffer, char *combo) {
    combo[0] = buffer[0];
}

void release(RING_BUFFER *r) {
    free(r->buffer);
    free(r->combo);
}

void prepare_ring(void) {
    int i;
    clear();
    for(i = 0; i < ring_count; ++i) {
        push_back(BELL_LF, BELL_LF_SIZE);
    }

}

void show_ring(void) {
    int item, k;
    int size = queue_size();

    item = queue_begin;
    for(k = 0; k < size; k++) {
        if(! strcmp(BELL_LF, queue[item]) ) {
            continue;
        } else {
            fputs(queue[item], stdout);
        }

        item = next(item);
    }

}
